﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Service.BoardService;
using System.ComponentModel;
using Service;
using com.google.zxing.common;
using com.google.zxing.qrcode;
using System.Drawing;
using System.Windows.Interop;
using Common.Logger;

namespace AnzeigenApp
{
    /// <summary>
    /// Interaktionslogik für EigeneAnzeigeView.xaml
    /// </summary>
    public partial class EigeneAnzeigeDetailView : UserControl, INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        //private string guid;
        private Anzeige anzeige;
        private FilterAnzeigenView filterView;

        public EigeneAnzeigeDetailView(Anzeige anzeige, FilterAnzeigenView filterView)
        {
            Logger.LogEvent(LogEvent.EigenesInseratAngeschaut, filterView.Model.Guid);
            InitializeComponent();
            images = new ObservableCollection<ImageSource>();
            this.anzeige = anzeige;
            DataContext = this;
            this.filterView = filterView;
            UpdateData(anzeige);
        }

        private ObservableCollection<ImageSource> images;

        public ObservableCollection<ImageSource> Images
        {
            get
            {
                return images;
            }
            set
            {
                if (images != value)
                {
                    images = value;
                    //this.OnPropertyChanged("Images");
                }
            }
        }

        private String changePublishStateButtonText;

        public String ChangePublishStateButtonText
        {
            get
            {
                return changePublishStateButtonText;
            }
            set
            {
                if (changePublishStateButtonText != value)
                {
                    changePublishStateButtonText = value;
                    this.OnPropertyChanged("ChangePublishStateButtonText");
                }
            }
        }

        //private String changePublishStateButtonText;

        //public String ChangePublishStateButtonText
        //{
        //    get
        //    {
        //        return changePublishStateButtonText;
        //    }
        //    set
        //    {
        //        if (changePublishStateButtonText != value)
        //        {
        //            changePublishStateButtonText = value;
        //            this.OnPropertyChanged("ChangePublishStateButtonText");
        //        }
        //    }
        //}

        //private void UserControl_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
        //{
        //    images.Clear();
        //    Anzeige anzeige = e.NewValue as Anzeige;
        //    if (anzeige != null)
        //    {
        //        textBlock1.Text = anzeige.Titel;
        //        textBlock2.Text = anzeige.Beschreibung;
        //        textBlock3.Text = anzeige.Text;
        //        string tmpKategorie = "";
        //        foreach (KeyValuePair<int, string> kvp in anzeige.Kategorie)
        //            tmpKategorie += kvp.Value + "\n";
        //        textBlock3.Text = tmpKategorie;
        //        foreach (String path in anzeige.Images)
        //            images.Add(BitmapLoader.GetImageSource(path));

        //        //Image img1 = new Image();
        //        //Image img2 = new Image();
        //        //Image img3 = new Image();
        //        //img1.Source = src1;
        //        //img2.Source = src1;
        //        //img3.Source = src1;

        //        pics.ItemsSource = Images;
        //        //image1.Source = src;
        //    }
        //}

        private void UpdateData(Anzeige anzeige)
        {
            if (anzeige != null)
            {
                int maxSize = Common.AppConfHelper.GetValueAsInt("max_qr_code_size");
                images.Clear();
                txtText.Text = anzeige.Text;
                String tmp = String.Empty;
                foreach (var s in anzeige.Tags)
                    tmp += s + ", ";
                if (tmp.Length >= 2)
                    tmp = tmp.Substring(0, tmp.Length - 2);
                txtTags.Text = tmp;
                txtTitel.Text = anzeige.Titel;
                txtBeschreibung.Text = anzeige.Beschreibung;
                //string tmpKategorie = "";
                //foreach (KeyValuePair<int, string> kvp in anzeige.Kategorie)
                //    tmpKategorie += kvp.Value + "\n";
                //textBlock3.Text = tmpKategorie;
                foreach (String path in anzeige.Images)
                    images.Add(BitmapLoader.GetImageSource(path));
                if (images.Count() > 0)
                {
                    imgPreview.Source = images.First();
                    txtImgInfo.Visibility = Visibility.Visible;
                }
                ChangePublishStateButtonText = anzeige.Published ? "Anzeige auf privat setzen" : "Anzeige auf öffentlich setzen";
                //Image img1 = new Image();
                //Image img2 = new Image();
                //Image img3 = new Image();
                //img1.Source = src1;
                //img2.Source = src1;
                //img3.Source = src1;

                pics.ItemsSource = Images;

                String data = anzeige.ToJsonString(maxSize == 0 ? 300 : maxSize);
                ByteMatrix matrix;
                com.google.zxing.Writer writer = new QRCodeWriter();
                try
                {
                    matrix = writer.encode(data, com.google.zxing.BarcodeFormat.QR_CODE, 400, 400);
                    int width = matrix.Width;
                    int height = matrix.Height;

                    sbyte[][] array = matrix.Array;
                    Bitmap image = new Bitmap(width, height, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
                    for (int y = 0; y < height; y++)
                    {
                        for (int x = 0; x < width; x++)
                        {
                            int grayValue = array[y][x] & 0xff;
                            image.SetPixel(x, y, grayValue == 0 ? System.Drawing.Color.Black : System.Drawing.Color.White);
                        }
                    }
                    var src = Imaging.CreateBitmapSourceFromHBitmap(image.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromWidthAndHeight(width, height));
                    qrCode.Source = src;
                    //images.Add(src);
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.Message);
                }
            }
        }

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        private void SurfaceButton_Click(object sender, RoutedEventArgs e)
        {
            Logger.LogEvent(LogEvent.InseratGelöscht, filterView.Model.Guid);
            Proxy fassade = new Proxy();
            fassade.AnzeigeLöschen(filterView.Model.Guid, anzeige.ID);
            filterView.UnfilteredAnzeigen.Remove(anzeige);
            filterView.SelectedAnzeige = null;
        }

        private void SurfaceButton_Click_1(object sender, RoutedEventArgs e)
        {
            Logger.LogEvent((anzeige.Published ? LogEvent.InseratPrivatGesetzt : LogEvent.InseratÖffentlichGesetzt), filterView.Model.Guid);
            Proxy fassade = new Proxy();
            fassade.AnzeigeSetÖffentlichStatus(filterView.Model.Guid, anzeige.ID, !anzeige.Published);
            Anzeige newAnzeige = fassade.GetAnzeige(filterView.Model.Guid, anzeige.ID);
            if (anzeige.Equals(newAnzeige))
                if (newAnzeige != null)
                {
                    this.anzeige = newAnzeige;
                    UpdateData(newAnzeige);
                    filterView.UnfilteredAnzeigen.Where(x => x.ID == anzeige.ID).Single().UpdateValues(anzeige);
                    //filterView.UnfilteredAnzeigen.Remove(anzeige);
                    //filterView.UnfilteredAnzeigen.Add(newAnzeige);
                }
        }

        private void SurfaceButton_Click_3(object sender, RoutedEventArgs e)
        {
            gridImages.Visibility = Visibility.Hidden;
        }

        private void imgPreview_TouchDown(object sender, TouchEventArgs e)
        {
            gridImages.Visibility = Visibility.Visible;
        }
    }
}
