﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FluentNHibernate.Mapping;
using System.Runtime.Serialization;
using ProfilKomponente;
using System.IO;

namespace AnzeigenKomponente
{
    public class Anzeige
    {
        public virtual int AccountID { get; private set; }
        public virtual int ID { get; private set; }
        public virtual String Titel { get; set; }
        public virtual String Beschreibung { get; set; }
        public virtual String Text { get; set; }
        public virtual DateTime Datum { get; set; }
        public virtual TimeSpan Restzeit { get; set; }
        public virtual IList<Tag> Tags { get; set; }
        public virtual IDictionary<int, Kategorie> Kategorie { get; set; }
        public virtual bool Published { get; set; }
        public virtual string Appearance { get; set; }
        public virtual IList<int> Images { get; set; }
        public virtual KontaktOptionen KontaktOptionen { get; set; }
        public virtual bool Gelöscht { get; set; }
        public virtual DateTime GelöschtAm { get; set; }

        public Anzeige() { }

        public Anzeige(int accountID, string titel, string beschreibung, string text, IList<Tag> tags,
            IDictionary<int, Kategorie> kategorie, TimeSpan restzeit, String appearance, IList<int> images, KontaktOptionen kontaktOptionen)
        {
            this.AccountID = accountID;
            this.Titel = titel;
            this.Beschreibung = beschreibung;
            this.Text = text;
            this.Tags = tags;
            this.Kategorie = kategorie;
            this.Datum = DateTime.Now;
            this.Restzeit = restzeit;
            this.Published = true;
            this.Appearance = appearance;
            this.Images = images;
            this.KontaktOptionen = kontaktOptionen;
            Gelöscht = false;
        }

        public virtual void SetPublished(bool value)
        {
            Published = value;
        }

        public override string ToString()
        {
            return Titel + Beschreibung + Text;
        }
    }

    public class AnzeigeMap : ClassMap<Anzeige>
    {
        public AnzeigeMap()
        {
            Id(x => x.ID);
            Map(x => x.AccountID);
            HasManyToMany(x => x.Tags).Not.LazyLoad();
            HasManyToMany(x => x.Images).Element("image_id").Table("ImageIDToAnzeige").Not.LazyLoad();
            //HasManyToMany(x => x.Kategorie).AsMap<int>("hierarchieStufe").Element("kategorie", c => c.Type<Kategorie>());
            HasManyToMany<Kategorie>(x => x.Kategorie).Table("KategorieZuweisung").AsMap<int>("hierarchieStufe").Not.LazyLoad();
            //References(x => x.Inhalt).LazyLoad(Laziness.False).Cascade.All();
            //References(x => x.Kategorie).LazyLoad(Laziness.False).Cascade.None();
            Map(x => x.Datum);
            Map(x => x.Restzeit);
            Map(x => x.Published);
            Map(x => x.Appearance);
            Map(x => x.Titel);
            Map(x => x.Beschreibung);
            Map(x => x.Text);
            Map(x => x.KontaktOptionen);
            Map(x => x.Gelöscht);
            Map(x => x.GelöschtAm);
        }
    }
}
