﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PersistenzManager;
using RechteVerwaltungsKomponente;
using BilderKomponente;
using ProfilKomponente;

namespace AnzeigenKomponente
{
    public class AnzeigenKomponente : IAnzeigenKomponente
    {
        private AnzeigenVerwalter verwalter = null;
        private IProfilKomponente profilKomponente;
        private BilderKomponente.BilderKomponente bilderKomponente;
        private int chunkSize;

        public AnzeigenKomponente(IPersistenz persistenz, BilderKomponente.BilderKomponente bilderKomponente, IProfilKomponente profilKomponente)
        {
            this.verwalter = new AnzeigenVerwalter(persistenz);
            this.bilderKomponente = bilderKomponente;
            this.profilKomponente = profilKomponente;
            chunkSize = Common.AppConfHelper.GetValueAsInt("chunk_size");
        }

        public TransportAnzeige AnzeigeErstellen(int accountID, string titel, string beschreibung,
            string text, IList<Tag> tags, IDictionary<int, int> kategorie, string appearance, IList<int> images, KontaktOptionen kontaktOptionen)
        {
            IList<Tag> realTags = new List<Tag>();
            IDictionary<int, Kategorie> realKategorie = new Dictionary<int, Kategorie>();
            foreach (Tag t in tags)
            {
                if (String.IsNullOrEmpty(t.Name))
                    continue;
                Tag tmpTag = verwalter.GetTag(t.Name.ToLower());
                if (tmpTag != null)
                    realTags.Add(tmpTag);
                else
                {
                    tmpTag = TagErstellen(t.Name.ToLower());
                    if (tmpTag != null)
                        realTags.Add(tmpTag);
                }
            }
            ICollection<int> hierarchieStufen = new List<int>();
            foreach (KeyValuePair<int, int> k in kategorie)
            {
                hierarchieStufen.Add(k.Key);
                Kategorie tmpKat = verwalter.GetKategorie(k.Value);
                if (tmpKat == null)
                    throw new Exception(String.Format("Kategorie mit der id {0} existiert nicht", k.Value));
                realKategorie.Add(k.Key, tmpKat);
            }
            if (hierarchieStufen.Count() < 1)
                throw new Exception("Es ist keine Kategorie ausgewählt");
            hierarchieStufen = hierarchieStufen.OrderBy(x => x).ToList();
            for (int i = 1; i <= hierarchieStufen.Count; i++)
                if (hierarchieStufen.ElementAt(i - 1) != i)
                    throw new Exception("Kategoriestufen müssen bei 1 beginnen und aufsteigend sein");


            IList<Kategorie> allKategorien = verwalter.GetAllKategorien();
            IList<Kategorie> kategorienStufe1 = new List<Kategorie>();
            foreach (Kategorie k in allKategorien.Take(allKategorien.Count))
                kategorienStufe1.Add(k);
            foreach (Kategorie k in allKategorien)
            {
                foreach (Kategorie kat in k.Unterkategorien)
                {
                    kategorienStufe1.Remove(kategorienStufe1.Where(x => x.Name.Equals(kat.Name)).SingleOrDefault());
                }
            }
            IList<Kategorie> kategorienStufe2 = new List<Kategorie>();
            IList<Kategorie> kategorienStufe3 = new List<Kategorie>();
            foreach (var k in kategorienStufe1)
                foreach (var k1 in k.Unterkategorien)
                    kategorienStufe2.Add(k1);

            foreach (var k in kategorienStufe2)
                foreach (var k1 in k.Unterkategorien)
                    kategorienStufe3.Add(k1);

            foreach (var e in realKategorie.Where(x => x.Key == 1).Select(x => x.Value))
            {
                if (!kategorienStufe1.Contains(e))
                    throw new Exception(String.Format("Die gewählte Stufe 1 Kategorie {0}, existiert nicht als Kategorie erster Stufe", e.Name));
            }

            foreach (var e in realKategorie.Where(x => x.Key == 2).Select(x => x.Value))
            {
                if (!kategorienStufe2.Contains(e))
                    throw new Exception(String.Format("Die gewählte Stufe 2 Kategorie {0}, existiert nicht als Kategorie zweiter Stufe", e.Name));
            }

            foreach (var e in realKategorie.Where(x => x.Key == 3).Select(x => x.Value))
            {
                if (!kategorienStufe3.Contains(e))
                    throw new Exception(String.Format("Die gewählte Stufe 3 Kategorie {0}, existiert nicht als Kategorie dritter Stufe", e.Name));
            }



            IList<Kategorie> unterKategorien = null;
            //Über die Kategorien iterieren, sortiert nach hierachiestufe, von 1 aufwärts
            foreach (int i in realKategorie.Keys.OrderBy(x => x))
            {
                Kategorie tmp;
                realKategorie.TryGetValue(i, out tmp);
                //für hierarchiestufe 1, ist unterKategorien noch null, also wird es nur gesetzt.
                //für hierarchiestufe 2, beinhaltet unterKategorien die unterKategorien von der Kategorie der Hierarchiestufe 1
                //also muss die aktuelle Kategorie dort enthalten sein, damit die hierarchie korrekt eingehalten wird, usw..
                if (unterKategorien != null)
                    if (!unterKategorien.Contains(tmp))
                        throw new Exception(String.Format("Fehler,Kategorie {0} für stufe {1}, ist nicht als unterKategorie der stufe davor vertreten", tmp.ID, i));
                unterKategorien = tmp.Unterkategorien;
            }
            Anzeige a = new Anzeige(accountID, titel, beschreibung, text, realTags, realKategorie, new TimeSpan(7, 0, 0, 0), appearance, bilderKomponente.SelectOwnImages(images, accountID), kontaktOptionen);
            verwalter.SaveAnzeige(a);
            return GetTransportAnzeige(a);
        }

        public IList<Tag> GetAllTags()
        {
            return verwalter.GetAllTags();
        }

        public Tag TagErstellen(string name)
        {
            Tag tag = new Tag(name);
            verwalter.SaveTag(tag);
            return tag;
        }

        public IList<Kategorie> GetAllKategorien()
        {
            return verwalter.GetAllKategorien();
        }


        public TransportAnzeige AnzeigeAktualisieren(int accountID, int anzeigeID, string titel, string beschreibung, string text, IList<Tag> tags,
            IDictionary<int, int> kategorie, string appearance, IList<int> images)
        {
            Anzeige anzeige = verwalter.GetAnzeige(anzeigeID);
            if (anzeige == null || anzeige.AccountID != accountID)
                return null;
            foreach (Tag t in anzeige.Tags)
                verwalter.DeleteTag(t);
            IList<Tag> realTags = new List<Tag>();
            IDictionary<int, Kategorie> realKategorie = new Dictionary<int, Kategorie>();
            foreach (Tag t in tags)
            {
                if (String.IsNullOrEmpty(t.Name))
                    continue;
                Tag tmpTag = verwalter.GetTag(t.Name.ToLower());
                if (tmpTag != null)
                    realTags.Add(tmpTag);
                else
                {
                    tmpTag = TagErstellen(t.Name.ToLower());
                    if (tmpTag != null)
                        realTags.Add(tmpTag);
                }
            }
            anzeige.Tags = realTags;
            foreach (KeyValuePair<int, int> k in kategorie)
            {
                Kategorie tmpKat = verwalter.GetKategorie(k.Value);
                if (tmpKat == null)
                    throw new Exception(String.Format("Kategorie mit der id {0} existiert nicht", k.Value));
                realKategorie.Add(k.Key, tmpKat);
            }
            IList<Kategorie> unterKategorien = null;
            //Über die Kategorien iterieren, sortiert nach hierachiestufe, von 1 aufwärts
            foreach (int i in realKategorie.Keys.OrderBy(x => x))
            {
                Kategorie tmp;
                realKategorie.TryGetValue(i, out tmp);
                //für hierarchiestufe 1, ist unterKategorien noch null, also wird es nur gesetzt.
                //für hierarchiestufe 2, beinhaltet unterKategorien die unterKategorien von der Kategorie der Hierarchiestufe 1
                //also muss die aktuelle Kategorie dort enthalten sein, damit die hierarchie korrekt eingehalten wird, usw..
                if (unterKategorien != null)
                    if (!unterKategorien.Contains(tmp))
                        throw new Exception(String.Format("Fehler,Kategorie {0} für stufe {1}, ist nicht als unterKategorie der stufe davor vertreten", tmp.ID, i));
                unterKategorien = tmp.Unterkategorien;
            }
            anzeige.Kategorie = realKategorie;
            anzeige.Titel = titel;
            anzeige.Beschreibung = beschreibung;
            anzeige.Text = text;
            anzeige.Appearance = appearance;
            anzeige.Images = images;
            verwalter.SaveAnzeige(anzeige);
            return GetTransportAnzeige(anzeige);
        }

        public bool AnzeigeSetÖffentlichStatus(int accountID, int anzeigeID, bool status, AccountType type)
        {
            Anzeige anzeige = verwalter.GetAnzeige(anzeigeID);
            if (anzeige != null && (type.Equals(AccountType.Admin) || anzeige.AccountID.Equals(accountID)))
            {
                anzeige.SetPublished(status);
                verwalter.SaveAnzeige(anzeige);
                return true;
            }
            return false;
        }

        public bool AnzeigeLöschen(int accountID, int anzeigeID, AccountType type)
        {
            Anzeige anzeige = verwalter.GetAnzeige(anzeigeID);
            if (anzeige != null && (type.Equals(AccountType.Admin) || anzeige.AccountID.Equals(accountID)))
            {
                anzeige.Gelöscht = true;
                anzeige.GelöschtAm = DateTime.Now;
                verwalter.SaveAnzeige(anzeige);
                return true;
            }
            return false;
        }

        public IList<TransportAnzeige> GetAllAnzeigen(AccountType type, int chunk)
        {
            //wenn admin, dann alle, sonst nur öffentliche
            //Console.WriteLine("In AnzeigenKomponente:GetAllAnzeigen-AccountType:{0}-chunk:{1}", type, chunk);
            if (type == AccountType.Admin)
                return GetTransportAnzeigen(verwalter.GetAllAnzeigen(chunk, chunkSize));
            else
                return GetTransportAnzeigen(verwalter.GetAllPublishedAnzeigen(chunk, chunkSize));
        }

        public IList<TransportAnzeige> GetALLOwnAnzeigen(int accountID, int chunk)
        {
            return GetTransportAnzeigen(verwalter.GetAllAnzeigen(accountID, chunk, chunkSize));
        }

        public TransportAnzeige GetAnzeige(int accountID, int anzeigeID, AccountType type)
        {
            Anzeige anzeige = verwalter.GetAnzeige(anzeigeID);
            if (anzeige == null)
                return null;
            if (type.Equals(AccountType.Admin) || anzeige.AccountID.Equals(accountID))
                return GetTransportAnzeige(anzeige);
            else if (anzeige.Published)
                return GetTransportAnzeige(anzeige);
            return null;
        }

        public IList<TransportAnzeige> GetAllAnzeigenFromUser(int ownAccountID, int userAccountID, AccountType type, int chunk)
        {
            IList<Anzeige> tmpList = verwalter.GetAllAnzeigen(userAccountID, chunk, chunkSize);
            if (type == AccountType.Admin || ownAccountID == userAccountID)
            {
                //wenn man admin ist, oder der gesuchte user, darf man alle, so wie getAllOwn
                return GetTransportAnzeigen(verwalter.GetAllAnzeigen(ownAccountID, chunk, chunkSize));
            }
            else
            {
                //ansonstent ist weder admin, noch der gesuchte user, also darf man nur die öffentlichen anzeigen des users sehen
                return GetTransportAnzeigen(verwalter.GetAllPublishedAnzeigen(userAccountID, chunk, chunkSize));
            }
        }

        private TransportAnzeige GetTransportAnzeige(Anzeige anzeige)
        {
            string nickname = profilKomponente.GetNickname(anzeige.AccountID);
            IList<string> images = bilderKomponente.GetVirtualImagePaths(anzeige.Images);
            var profil = profilKomponente.GetProfil(anzeige.AccountID);
            return new TransportAnzeige(anzeige, nickname, images, profil.KontaktDetails);
        }

        private IList<TransportAnzeige> GetTransportAnzeigen(IList<Anzeige> anzeigen)
        {
            IList<TransportAnzeige> tmp = new List<TransportAnzeige>();
            foreach (Anzeige a in anzeigen)
                tmp.Add(GetTransportAnzeige(a));
            return tmp;
        }


        public bool IstOwnAnzeige(int accountID, int anzeigeID)
        {
            Anzeige anzeige = verwalter.GetAnzeige(anzeigeID);
            if (anzeige == null)
                return false;
            return anzeige.AccountID == accountID;
        }


        public bool SendeAnzeigePerMail(int accountID, string empfänger, int anzeigeID, AccountType type)
        {
            Anzeige anzeige = verwalter.GetAnzeige(anzeigeID);
            if (anzeige == null)
                return false;
            if (type.Equals(AccountType.Admin) || anzeige.AccountID.Equals(accountID))
                return MailKomponente.MailKomponente.SendHtmlMail(empfänger, "betreff", GetTransportAnzeige(anzeige).toHTMLString());
            else if (anzeige.Published)
                return MailKomponente.MailKomponente.SendHtmlMail(empfänger, "betreff", GetTransportAnzeige(anzeige).toHTMLString());
            return false;
        }

        public Kategorie ErstelleKategorie(string name, IList<string> unterkategorien)
        {
            IList<Kategorie> unterKats = new List<Kategorie>();
            foreach (string s in unterkategorien){
                var kk = verwalter.GetKategorie(s);
                if (kk == null)
                    unterKats.Add(new Kategorie(s));
                else
                    unterKats.Add(kk);
            }
            var kat = verwalter.GetKategorie(name);
            if (kat == null)
                kat = new Kategorie(name, unterKats);
            else
                foreach (var k in unterKats)
                    kat.Unterkategorien.Add(k);
            foreach (var k in unterKats)
                verwalter.SaveKategorie(k);
            verwalter.SaveKategorie(kat);
            return kat;
        }

        public bool LöscheKategorie(int id)
        {
            var kat = verwalter.GetKategorie(id);
            if (kat != null)
            {
                foreach (var k in verwalter.GetAllKategorien())
                {
                    if (k.Unterkategorien.Contains(kat))
                    {
                        k.Unterkategorien.Remove(kat);
                    }
                    verwalter.SaveKategorie(k);
                }
                verwalter.DeleteKategorie(kat);
            }
            return true;
        }
    }
}
