﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PersistenzManager;

namespace AnzeigenKomponente
{
    public class AnzeigenVerwalter
    {
        private IPersistenz persistenz = null;

        public AnzeigenVerwalter(IPersistenz persistenz)
        {
            this.persistenz = persistenz;
        }

        public IList<Anzeige> GetAllAnzeigen(int chunk, int chunksize)
        {
            return persistenz.LinqQuery<Anzeige>().Where(x => !x.Gelöscht).Skip(chunksize * chunk).Take(chunksize).ToList();
        }

        public IList<Anzeige> GetAllPublishedAnzeigen(int chunk, int chunksize)
        {
            return (from a in persistenz.LinqQuery<Anzeige>()
                    where (a.Published) && (!a.Gelöscht)
                    select a).Skip(chunksize * chunk).Take(chunksize).ToList();
        }

        public IList<Anzeige> GetAllPublishedAnzeigen(int accountID, int chunk, int chunksize)
        {
            return (from a in persistenz.LinqQuery<Anzeige>()
                    where (a.Published) && (a.AccountID.Equals(accountID)) && (!a.Gelöscht)
                    select a).Skip(chunksize * chunk).Take(chunksize).ToList();
        }

        public IList<Anzeige> GetAllAnzeigen(int accountID, int chunk, int chunksize)
        {
            return persistenz.LinqQuery<Anzeige>().Where(x => x.AccountID.Equals(accountID) && (!x.Gelöscht)).Skip(chunksize * chunk).Take(chunksize).ToList();
        }

        public IList<Kategorie> GetAllKategorien()
        {
            return persistenz.GetAll<Kategorie>();
        }

        public IList<Tag> GetAllTags()
        {
            return persistenz.GetAll<Tag>();
        }

        public void SaveTag(Tag tag)
        {
            persistenz.Save<Tag>(tag);
        }

        public Anzeige GetAnzeige(int anzeigeID)
        {
            return (from a in persistenz.LinqQuery<Anzeige>()
                    where a.ID.Equals(anzeigeID) && (!a.Gelöscht)
                    select a).SingleOrDefault<Anzeige>();
        }

        public void SaveAnzeige(Anzeige anzeige)
        {
            persistenz.Save<Anzeige>(anzeige);
        }

        public void DeleteAnzeige(Anzeige anzeige)
        {
            persistenz.Delete<Anzeige>(anzeige);
        }

        public void DeleteTag(Tag tag)
        {
            persistenz.Delete<Tag>(tag);
        }

        public Tag GetTag(string name)
        {
            return (from a in persistenz.LinqQuery<Tag>()
                    where a.Name.Equals(name)
                    select a).SingleOrDefault<Tag>();
        }

        public Kategorie GetKategorie(int id)
        {
            return (from a in persistenz.LinqQuery<Kategorie>()
                    where a.ID.Equals(id)
                    select a).SingleOrDefault<Kategorie>();
        }

        public Kategorie GetKategorie(string name)
        {
            return persistenz.GetAll<Kategorie>().Where(x => x.Name.ToLower().Equals(name.ToLower())).SingleOrDefault();
        }

        public void DeleteKategorie(Kategorie kat)
        {
            persistenz.Delete<Kategorie>(kat);
        }

        public void SaveKategorie(Kategorie kat)
        {
            persistenz.Save<Kategorie>(kat);
        }
    }
}
