﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;

namespace Common.Logger
{
    public class Logger
    {
        private static bool isRunning = false;
        private static List<LogEventEntry> logEvents;
        private static StreamWriter logFile;
        public static String fullPattern = Regex.Replace(DateTimeFormatInfo.CurrentInfo.FullDateTimePattern, "(:ss|:s)", "$1.fff");
        private static String fileName;
        public static object synchronizer = new object();

        public static void Start()
        {
            fileName = String.Format("logFile-{0}.txt", DateTime.Now.Ticks);
            File.Create(fileName).Close();
            logEvents = new List<LogEventEntry>();
            isRunning = true;
            new Task(() => run()).Start();
        }

        public static string GetActualLogfileName()
        {
            return fileName;
        }

        public static void StartNewLogfile()
        {
            try
            {
                lock (synchronizer)
                {
                    LogEventEntry[] entries;
                    lock (logEvents)
                    {
                        entries = new LogEventEntry[logEvents.Count];
                        logEvents.CopyTo(entries);
                        logEvents.Clear();
                    }

                    logFile = openLogFile();
                    foreach (var e in entries)
                        logFile.WriteLine(e);
                    logFile.Flush();
                    logFile.Close();

                    fileName = String.Format("logFile-{0}.txt", DateTime.Now.Ticks);
                    File.Create(fileName).Close();
                }
            }
            catch (Exception e) { }
        }

        public static void Stop()
        {
            if (logFile != null)
                logFile.Close();
            isRunning = false;
        }

        private static StreamWriter openLogFile()
        {
            return File.AppendText(fileName);
        }

        private static void run()
        {
            while (isRunning)
            {
                try
                {
                    lock (synchronizer)
                    {
                        LogEventEntry[] entries;
                        lock (logEvents)
                        {
                            entries = new LogEventEntry[logEvents.Count];
                            logEvents.CopyTo(entries);
                            logEvents.Clear();
                        }

                        logFile = openLogFile();
                        foreach (var e in entries)
                            logFile.WriteLine(e);
                        logFile.Flush();
                        logFile.Close();
                    }  
                }
                catch (Exception e) { }
                Thread.Sleep(5000);
            }
        }

        public static void LogEvent(LogEvent logEvent, string guid)
        {
            lock (synchronizer)
            {
                if (logEvents != null)
                    new Task(() =>
                        {
                            lock (logEvents)
                            {
                                logEvents.Add(new LogEventEntry() { LogEvent = logEvent, Date = DateTime.Now, Guid = guid });
                            }
                        }
                    ).Start();
            }
        }

        public static void SaveLogfile(Stream fileStream)
        {
            try
            {
                var dir = Directory.CreateDirectory("log");
                byte[] buffer = new byte[1024];
                int read = 0;
                string filePath = String.Format("log\\logFile-{0}.txt", DateTime.Now.Ticks);
                FileStream outputStream = new FileStream(filePath, FileMode.Create);

                buffer = new byte[1024];
                while ((read = fileStream.Read(buffer, 0, buffer.Length)) != 0)
                    outputStream.Write(buffer, 0, read);

                fileStream.Close();
                outputStream.Close();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                Console.WriteLine(e.StackTrace);
            }
        }
    }
}
