﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PersistenzManager;
using RechteVerwaltungsKomponente;
using System.ServiceModel;
using AnzeigenKomponente;
using System.IO;
using ProfilKomponente;
using FeedbackKomponente;
using Common;

namespace Fassade
{
    public class ÖffentlicheFassade : IÖffentlicheFassade
    {
        public IPersistenz persistenz = null;
        public IConversationFactory conversationFactory = null;

        public RechteVerwaltungsKomponente.RechteVerwaltung rechteVerwaltung = null;
        public AccountVerwaltungsKomponente.AccountVerwaltung accountVerwaltung = null;
        public IAnzeigenKomponente anzeigenKomponente = null;
        public MailKomponente.MailKomponente mailKomponente;
        public BilderKomponente.BilderKomponente bilderKomponente;
        public ProfilKomponente.ProfilKomponente profilKomponente;
        public FeedbackKomponente.FeedbackKomponente feedbackKomponente;
        public StatusKomponente.StatusKomponente statusKomponente;

        public ÖffentlicheFassade(IPersistenz persistenz, IConversationFactory conversationFactory)
        {
            this.persistenz = persistenz;
            this.conversationFactory = conversationFactory;
            profilKomponente = new ProfilKomponente.ProfilKomponente(persistenz);
            rechteVerwaltung = new RechteVerwaltungsKomponente.RechteVerwaltung(persistenz);
            accountVerwaltung = new AccountVerwaltungsKomponente.AccountVerwaltung(persistenz, rechteVerwaltung);
            bilderKomponente = new BilderKomponente.BilderKomponente(persistenz);
            mailKomponente = new MailKomponente.MailKomponente(rechteVerwaltung, bilderKomponente);
            this.anzeigenKomponente = new AnzeigenKomponente.AnzeigenKomponente(persistenz, bilderKomponente, profilKomponente);
            feedbackKomponente = new FeedbackKomponente.FeedbackKomponente(persistenz);
            statusKomponente = new StatusKomponente.StatusKomponente();
            statusKomponente.Start();
            //mailKomponene.Start();
        }

        public string Registrieren(string email, string passwort)
        {
            string tmp = "";
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.RegistrierungsVersuch, "");
                        int id;
                        bool exists;
                        email = email.ToLower();
                        email = email.Trim();
                        tmp = accountVerwaltung.Registrieren(email, passwort, !AppConfHelper.GetValueAsBool("aktivierungsmail"), false, out id, out exists);
                        if (!String.IsNullOrEmpty(tmp) && !exists)
                        {
                            profilKomponente.ProfilErstellen(id, email);
                            Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.RegistrierungsErfolg, tmp);
                        }
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public string Login(string email, string passwort)
        {
            string ticket = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        email = email.ToLower();
                        ticket = rechteVerwaltung.Login(email, passwort);
                        Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.LoginAusgeführt, ticket);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return ticket;
        }

        public bool AccountAktivieren(string guid, string code)
        {
            bool tmp = false;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("AccountAktivieren for {0}", guid);
                        AccountType type;
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.InaktivUserFall, guid, out type);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = accountVerwaltung.Aktivieren(id, code);
                        if (tmp && type == AccountType.Inaktiv)
                        {
                            profilKomponente.ChangeToSichtbar(id);
                            Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.AccountAktiviert, guid);
                        }
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public string GetAccountMail(string guid)
        {
            string tmp = "";
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        tmp = rechteVerwaltung.GetAccountMail(guid);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public bool SendeAktivierungsmail(string guid)
        {
            bool tmp = false;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("SendeAktivierungsmail fuer {0}", guid);
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.InaktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = accountVerwaltung.SendeAktivierungsmail(id, guid);
                        Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.SendeAktivierungsMail, guid);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public bool IstAccountAktiviert(string guid)
        {
            bool tmp = false;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("IstAccountAktiviert");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.InaktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = rechteVerwaltung.IstAccountAktiviert(guid);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public Profil GetProfil(string guid)
        {
            Profil tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetProfil");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = profilKomponente.GetProfil(id);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public Profil GetProfilByRegisteredMail(string guid, string registeredEmail)
        {
            Profil tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetProfilByRegisteredMail");
                        AccountType type;
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = profilKomponente.GetProfil(registeredEmail, id, type);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public IList<ProfilVorschau> GetAlleProfile(string guid)
        {
            IList<ProfilVorschau> tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetAlleProfile");
                        AccountType type;
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = profilKomponente.GetAlleProfile(type);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public Profil ProfilAktualisieren(string guid, string nickname, string vorname, string nachname, bool nameVisible,
            string strasse, string plz, string stadt, bool adresseVisible, string telefon, bool telefonVisibible,
            string email, bool emailVisible, int profilbildImageID, bool sichtbar)
        {
            Profil tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("ProfilAktualisieren");
                        AccountType type;
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                        {
                            string imgPath = "";
                            if (profilbildImageID == -1)
                                imgPath = "keep";
                            else
                                imgPath = bilderKomponente.GetVirtualImagePath(profilbildImageID, id);

                            tmp = profilKomponente.ProfilAktualisieren(id, nickname, vorname, nachname, nameVisible, strasse, plz, stadt, adresseVisible, telefon,
                                telefonVisibible, email, emailVisible, imgPath, sichtbar);
                        }
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public TransportAnzeige AnzeigeErstellen(string guid, string titel, string beschreibung, string text, IList<Tag> tags,
            IDictionary<int, int> kategorie, string appearance, IList<int> images, KontaktOptionen kontaktOptionen = KontaktOptionen.Öffentliche)
        {
            TransportAnzeige tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.AnzeigeErstellenVersuch, guid);
                        Console.WriteLine("AnzeigeErstellen");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = anzeigenKomponente.AnzeigeErstellen(id, titel, beschreibung, text, tags, kategorie, appearance, images, kontaktOptionen);
                        if (tmp != null)
                            Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.AnzeigeErstellenErfolg, guid);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public TransportAnzeige AnzeigeAktualisieren(string guid, int anzeigeID, string titel, string beschreibung, string text, IList<Tag> tags,
            IDictionary<int, int> kategorie, string appearance, IList<int> images)
        {
            TransportAnzeige tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("AnzeigeAktualisieren");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = anzeigenKomponente.AnzeigeAktualisieren(id, anzeigeID, titel, beschreibung, text, tags, kategorie, appearance, images);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public bool AnzeigeSetÖffentlichStatus(string guid, int anzeigeID, bool status)
        {
            bool tmp = false;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("AnzeigeSetÖffentlichStatus");
                        AccountType type;
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = anzeigenKomponente.AnzeigeSetÖffentlichStatus(id, anzeigeID, status, type);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public bool AnzeigeLöschen(string guid, int anzeigeID)
        {
            bool tmp = false;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("AnzeigeLöschen");
                        AccountType type;
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = anzeigenKomponente.AnzeigeLöschen(id, anzeigeID, type);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public IList<Tag> GetAllTags()
        {
            IList<Tag> tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        tmp = anzeigenKomponente.GetAllTags();
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public IList<Kategorie> GetAllKategorien()
        {
            IList<Kategorie> tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        tmp = anzeigenKomponente.GetAllKategorien();
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public IList<TransportAnzeige> GetAllAnzeigen(string guid, int chunk)
        {
            Console.WriteLine("In Fassade:GetAllAnzeigen-Guid:{0}-chunk:{1}", guid, chunk);
            IList<TransportAnzeige> tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        AccountType type;
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        Console.WriteLine("In Fassade:IsAccessPermitted:{0}-AccountType:{1}", id, type);
                        if (id > -1)
                            tmp = anzeigenKomponente.GetAllAnzeigen(type, chunk);
                    }));
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in Fassade GetAllAnzeigen:{0}", ex.Message);
            }
            return tmp;
        }

        public IList<TransportAnzeige> GetOwnAnzeigen(string guid, int chunk)
        {
            IList<TransportAnzeige> tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetOwnAnzeigen");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = anzeigenKomponente.GetALLOwnAnzeigen(id, chunk);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public TransportAnzeige GetAnzeige(string guid, int anzeigeID)
        {
            TransportAnzeige tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetAnzeige");
                        AccountType type;
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = anzeigenKomponente.GetAnzeige(id, anzeigeID, type);
                        if (tmp != null)
                            Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.AnzeigeAbgerufen, guid);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public bool SendeAnzeigePerMail(string guid, int anzeigeID)
        {
            bool tmp = false;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("SendeAnzeigePerMail");
                        AccountType type;
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        Console.WriteLine("id {0}", id);
                        string empfänger = rechteVerwaltung.GetAccountMail(guid);
                        if (id > -1)
                            tmp = anzeigenKomponente.SendeAnzeigePerMail(id, empfänger, anzeigeID, type);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public IList<TransportAnzeige> GetAllAnzeigenFromUser(string guid, string nickname, int chunk)
        {
            IList<TransportAnzeige> tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetAllAnzeigenFromUser");
                        AccountType type;
                        int userID = profilKomponente.GetAccountID(nickname);
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = anzeigenKomponente.GetAllAnzeigenFromUser(id, userID, type, chunk);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public FileTransferResult ImageErstellen(FileTransferRequest file)
        {
            FileTransferResult tmp = new FileTransferResult();
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("ImageErstellen");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, file.guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp.imageID = bilderKomponente.SaveImage(file.Data, id);
                        if (tmp.imageID != -1)
                            Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.BildHochgeladen, file.guid);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public string GetImageHash(string guid, int imageID)
        {
            string tmp = "";
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetImageHash");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = bilderKomponente.GetImageHash(imageID, id);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public string GetImagePath(string guid, int imageID)
        {
            string tmp = "";
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetImagePath");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = bilderKomponente.GetVirtualImagePath(imageID, id);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public IList<string> GetImagePaths(string guid, IList<int> imageIDs)
        {
            IList<string> tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetImagePaths");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = bilderKomponente.GetVirtualImagePaths(imageIDs, id);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }


        public bool IstOwnAnzeige(string guid, int anzeigeID)
        {
            bool tmp = false;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("IstOwnAnzeige");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = anzeigenKomponente.IstOwnAnzeige(id, anzeigeID);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public bool PostFeedback(string guid, string text)
        {
            bool tmp = false;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("PostFeedback");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = feedbackKomponente.PostFeedback(text);
                        if (tmp)
                            Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.FeedbackAbgeschickt, guid);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public IList<Feedback> GetFeedbackList(string guid, int chunk)
        {
            IList<Feedback> tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetFeedbackList");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AdminFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = feedbackKomponente.GetFeedbackList(chunk);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }


        public Profil GetProfilByID(string guid, int profileAccountID)
        {
            Profil tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("GetProfilByID");
                        AccountType type;
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = profilKomponente.GetProfil(profileAccountID, id, type);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public void IsAlive(string guid)
        {
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("IsAlive");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.BoardFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            statusKomponente.IsAlive();
                    }));
                }
            }
            catch (Exception ex)
            {
            }
        }

        public bool Status()
        {
            return statusKomponente.Status();
        }


        public void UploadLogfile(FileTransferRequest file)
        {
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("UploadLogfile");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.BoardFall, file.guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            Common.Logger.Logger.SaveLogfile(file.Data);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
        }

        public bool ChangePassword(string guid, string guidToChange, string oldPW, string newPW)
        {
            bool tmp = false;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("ChangePassword");
                        AccountType type;
                        rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guid, out type);
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AktivUserFall, guidToChange);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = rechteVerwaltung.ChangePassword(id, oldPW, newPW, type);
                        if (tmp)
                            Common.Logger.Logger.LogEvent(Common.Logger.LogEvent.PasswortGeändert, guid);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }


        public bool DeleteKategorie(string guid, int kategorieID)
        {
            bool tmp = false;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("DeleteKategorie");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AdminFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = anzeigenKomponente.LöscheKategorie(kategorieID);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }


        public Kategorie CreateKategorie(string guid, string name, IList<string> unterkategorien)
        {
            Kategorie tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        Console.WriteLine("CreateKategorie");
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AdminFall, guid);
                        Console.WriteLine("id {0}", id);
                        if (id > -1)
                            tmp = anzeigenKomponente.ErstelleKategorie(name, unterkategorien);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public string GetGuid(string guid, string email)
        {
            string tmp = null;
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        int id = rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AdminFall, guid);
                        if (id > -1)
                            tmp = rechteVerwaltung.GetGuidFromAccount(email);
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public string CreateAccount(string guid, string email, string passwort, bool activated)
        {
            string tmp = "";
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        int id;
                        bool exists;
                        email = email.ToLower();
                        if (rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AdminFall, guid) > -1)
                        {
                            tmp = accountVerwaltung.Registrieren(email, passwort, true, true, out id, out exists);
                            if (!String.IsNullOrEmpty(tmp) && !exists)
                            {
                                profilKomponente.ProfilErstellen(id, email);
                                profilKomponente.ChangeToSichtbar(id);
                            }
                        }
                    }));
                }
            }
            catch (Exception ex)
            {
            }
            return tmp;
        }

        public void SendRundmail(string guid, string subject, string text)
        {
            try
            {
                using (var conversation = conversationFactory.NewConversation())
                {
                    conversation.ExecuteTransactional(new TransactionalCode(() =>
                    {
                        if (rechteVerwaltung.IsAccessPermitted(Anwendungsfall.AdminFall, guid) > -1)
                        {
                            accountVerwaltung.SendRundmail(subject, text);
                        }
                    }));
                }
            }
            catch (Exception ex)
            {
            }
        }
    }
}
