﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Surface.Presentation.Controls;
using System.Collections.ObjectModel;
using LoginRegisterApp;
using AnzeigenApp;
using Service;
using BasisKlassen;
using System.ComponentModel;
using System.Threading.Tasks;
using System.Windows.Threading;
using System.Threading;
using Common.Logger;
using System.IO;
using Service.BoardService;

namespace MainApplication
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : SurfaceWindow, INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        LoginRegisterApp.Model loginRegisterAppModel;
        AnzeigenApp.Model anzeigenAppModel;
        ProfilApp.Model profilAppModel;
        InfoApp.Model infoAppModel;
        private bool idleCheckerIsRunning = false;
        private DateTime lastTouchEvent = DateTime.Now;
        private bool isIdle = false;
        private Timer heartbeatTimer, logfileUploader;

        public MainWindow()
        {
            InitializeComponent();
            DataContext = this;
            Logger.Start();
            loginRegisterAppModel = new LoginRegisterApp.Model(this);
            anzeigenAppModel = new AnzeigenApp.Model(this);
            profilAppModel = new ProfilApp.Model(this);
            infoAppModel = new InfoApp.Model(this);
            loginRegisterAppModel.Views.CollectionChanged += MenuCollectionChanged;
            views = new ObservableCollection<MyUserControl>();
            foreach (MyUserControl c in loginRegisterAppModel.Views)
                views.Add(c);
            loginRegisterAppModel.PropertyChanged += GuidChanged;

            anzeigenAppModel.Views.CollectionChanged += MenuCollectionChanged;
            foreach (MyUserControl c in anzeigenAppModel.Views)
                views.Add(c);

            profilAppModel.Views.CollectionChanged += MenuCollectionChanged;
            foreach (MyUserControl c in profilAppModel.Views)
                views.Add(c);

            infoAppModel.Views.CollectionChanged += MenuCollectionChanged;
            foreach (MyUserControl c in infoAppModel.Views)
                views.Add(c);
            lblTitle.Content = "Nicht eingeloggt. Anonyme Benutzung.";
            Views = new ObservableCollection<MyUserControl>(views.OrderBy(x => x.Title));
            heartbeatTimer = new Timer(HeartbeatSender, null, 0, 10000);
            logfileUploader = new Timer(LogfileUploader, null, new TimeSpan(1,0,0), new TimeSpan(1, 0, 0));

            //IDLE CHECKER
            //idleCheckerIsRunning = true;
            //new Task(new Action(() => IdleChecker())).Start();
        }

        private void HeartbeatSender(object info)
        {
            try
            {
                new Proxy().IsAlive(String.Empty);
            }
            catch (Exception e) { }
        }

        private void LogfileUploader(object info)
        {
            try
            {
                lock (Logger.synchronizer)
                {
                    Stream stream = File.OpenRead(Logger.GetActualLogfileName());
                    new Proxy().UploadLogfile(new FileTransferRequest("", stream));
                    stream.Close();
                    Logger.StartNewLogfile();
                }
            }
            catch (Exception e) { }
        }

        private void IdleChecker()
        {
            while (idleCheckerIsRunning)
            {
                Thread.Sleep(1000);
                if ((DateTime.Now - lastTouchEvent) >= new TimeSpan(0, 0, 30))
                {
                    this.Dispatcher.Invoke(DispatcherPriority.Background, (Action)(() =>
                    {
                        if (!isIdle)
                        {
                            if (!String.IsNullOrEmpty(loginRegisterAppModel.Guid))
                                loginRegisterAppModel.Guid = String.Empty;
                            var view = Views.Cast<MyUserControl>().First(x => x.Title.Equals("INSERATE")) as DurchsuchenView;
                            menu.SelectedItem = view;
                            view.contentControl.Content = view.AlleAnzeigenView;
                            view.btnAlleAnzeigen.Background = view.buttonBackgroundPressed;
                            view.btnEigeneAnzeigen.Background = view.buttonBackgroundNotPressed;
                            view.AlleAnzeigenView.filterAnzeigenView.imgKat1Pressed.Visibility = Visibility.Hidden;
                            view.AlleAnzeigenView.filterAnzeigenView.imgKat2Pressed.Visibility = Visibility.Hidden;
                            view.AlleAnzeigenView.filterAnzeigenView.setKategorie1(null);
                            view.AlleAnzeigenView.filterAnzeigenView.SelectedAnzeige = null;
                            view.AlleAnzeigenView.filterAnzeigenView.FilterString = String.Empty;
                            view.AlleAnzeigenView.LoadData();
                            isIdle = true;
                            Logger.LogEvent(LogEvent.IdleModusGestartet, loginRegisterAppModel.Guid);
                            new Task(new Action(() => IdleScroller())).Start();
                        }

                        //Visibility vis = view.Visibility;
                        //mainContent.Content = view;

                    }));
                }
                if ((DateTime.Now - lastTouchEvent) >= new TimeSpan(0, 1, 0))
                {
                    var view = Views.Cast<MyUserControl>().First(x => x.Title.Equals("INSERATE")) as DurchsuchenView;
                    if (view != null)
                        view.AlleAnzeigenView.LoadData();
                    lastTouchEvent = DateTime.Now;
                }
            }
        }

        private void IdleScroller()
        {
            var view = Views.Cast<MyUserControl>().First(x => x.Title.Equals("INSERATE")) as DurchsuchenView;
            //var scroller = view.AlleAnzeigenView.filterAnzeigenView.listBoxFilteredAnzeigen.Template.FindName("Scroller", view.AlleAnzeigenView.filterAnzeigenView.listBoxFilteredAnzeigen) as SurfaceScrollViewer;
            var scroller = view.AlleAnzeigenView.filterAnzeigenView.Scroller;
            int direction = 1;
            while (isIdle)
            {
                if (scroller != null)
                    this.Dispatcher.Invoke(DispatcherPriority.Background, (Action)(() =>
                    {
                        if (scroller.HorizontalOffset <= 0)
                        {
                            direction = 1;
                        }
                        if (scroller.HorizontalOffset >= scroller.ScrollableWidth)
                        {
                            scroller.ScrollToHorizontalOffset(scroller.ScrollableWidth);
                            if (direction == 1)
                                direction = -1;
                        }
                        scroller.ScrollToHorizontalOffset(scroller.HorizontalOffset + direction * 1);
                    }));

                else
                    scroller = view.AlleAnzeigenView.filterAnzeigenView.Scroller;
                Thread.Sleep(10);
            }
        }

        private void MenuCollectionChanged(object obj, System.Collections.Specialized.NotifyCollectionChangedEventArgs args)
        {
            switch (args.Action)
            {
                case System.Collections.Specialized.NotifyCollectionChangedAction.Add:
                    try
                    {
                        foreach (MyUserControl o in args.NewItems.Cast<MyUserControl>())
                            views.Add(o);
                    }
                    catch (Exception ex) { }
                    break;
                case System.Collections.Specialized.NotifyCollectionChangedAction.Remove:
                    try
                    {
                        foreach (MyUserControl o in args.OldItems.Cast<MyUserControl>())
                            views.Remove(o);
                    }
                    catch (Exception ex) { }
                    break;
            }
            Views = new ObservableCollection<MyUserControl>(views.OrderBy(x => x.Title));
        }

        private void GuidChanged(object obj, System.ComponentModel.PropertyChangedEventArgs args)
        {
            var model = obj as LoginRegisterApp.Model;
            if (model != null)
            {
                anzeigenAppModel.Guid = model.Guid;
                profilAppModel.Guid = model.Guid;
                infoAppModel.Guid = model.Guid;
                try
                {
                    var proxy = new Proxy();
                    var profil = proxy.GetProfil(model.Guid);
                    if (profil != null)
                        lblTitle.Content = new TextBlock() { Text = String.Format("Eingeloggt als {0}", profil.RegisteredEmail) };
                    else
                        lblTitle.Content = new TextBlock() { Text = "Nicht eingeloggt. Anonyme Benutzung." };
                }
                catch (Exception ex) { }
            }
        }

        private ObservableCollection<MyUserControl> views;

        public ObservableCollection<MyUserControl> Views
        {
            get
            {
                return views;
            }
            set
            {
                if (Views != value)
                {
                    views = value;
                    this.OnPropertyChanged("Views");
                }
            }
        }

        private void SurfaceWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            //AnzeigenUpdater.Stop();
            LogfileUploader(null);
            Logger.Stop();
            idleCheckerIsRunning = false;
        }

        private void menu_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var list = sender as SurfaceListBox;
            if (list.SelectedIndex == -1)
                startView.Visibility = Visibility.Visible;
            else
                startView.Visibility = Visibility.Hidden;
        }

        private void ListViewItem_PreviewMouseLeftButtonDown(object sender, TouchEventArgs e)
        {
            var view = (sender as ListBoxItem).Content as DurchsuchenView;
            if (view != null)
            {
                //view.contentControl.Content = view.AlleAnzeigenView;
                //view.btnAlleAnzeigen.Background = view.buttonBackgroundPressed;
                //view.btnEigeneAnzeigen.Background = view.buttonBackgroundNotPressed;
                //view.AlleAnzeigenView.filterAnzeigenView.imgKat1Pressed.Visibility = Visibility.Hidden;
                //view.AlleAnzeigenView.filterAnzeigenView.imgKat2Pressed.Visibility = Visibility.Hidden;
                //view.AlleAnzeigenView.filterAnzeigenView.setKategorie1(null);
                view.AlleAnzeigenView.filterAnzeigenView.SelectedAnzeige = null;
                //view.AlleAnzeigenView.filterAnzeigenView.FilterString = String.Empty;
                view.AlleAnzeigenView.LoadData();
            }
        }


        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        private void SurfaceWindow_PreviewTouchDown_1(object sender, TouchEventArgs e)
        {
            if (isIdle)
                Logger.LogEvent(LogEvent.IdleModusBeendet, loginRegisterAppModel.Guid);
            lastTouchEvent = DateTime.Now;
            isIdle = false;
            //if (menu.SelectedItem != null)
            //{
            //    Binding b = new Binding("SelectedItem");
            //    b.Source = menu;
            //    mainContent.SetBinding(ContentControl.ContentProperty, b);
            //}
        }

        private void SurfaceWindow_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (isIdle)
                Logger.LogEvent(LogEvent.IdleModusBeendet, loginRegisterAppModel.Guid);
            lastTouchEvent = DateTime.Now;
            isIdle = false;
        }


        //private ObservableCollection<String> views;

        //public ObservableCollection<String> Views
        //{
        //    get
        //    {
        //        return views;
        //    }
        //    set
        //    {
        //        if (Views != value)
        //        {
        //            views = value;
        //            //this.OnPropertyChanged("Views");
        //        }
        //    }
        //}
    }
}
