﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NHibernate;
using FluentNHibernate.Cfg;
using FluentNHibernate.Cfg.Db;
using NHibernate.Cfg;
using NHibernate.Tool.hbm2ddl;
using System.IO;
using NHibernate.Criterion;
using System.Data;
using System.Threading;
using NHibernate.Linq;

namespace PersistenzManager
{
    public class Persistenz : IPersistenz,IConversationFactory
    {
        private ISessionFactory sessionFactory = null;

        public static readonly Object Transaction = new Object();

        private FluentConfiguration config = null;
        private DBTyp typ;
        private DBStartMode mode;
        private string database;

        private static string DbFile = "bsp.db";

        public Persistenz(DBTyp typ, DBStartMode mode)
            : this(typ, mode, "bg")
        {
        }

        public Persistenz(DBTyp typ, DBStartMode mode, string database)
        {
            this.database = database;
            this.typ = typ;
            this.mode = mode;
            config = createSessionFacotry(typ, mode);
        }

        private FluentConfiguration createSessionFacotry(DBTyp typ, DBStartMode mode)
        {
            switch (typ)
            {
                case DBTyp.MSSQL2005: return Fluently.Configure()
                         .Database(MsSqlConfiguration.MsSql2005.ConnectionString(c => c
                             .Server("localhost\\SQLSERVER")
                             .Database(database)
                             .Username("")
                             .Password("")).IsolationLevel(IsolationLevel.ReadCommitted));
                case DBTyp.MSSQL2008: return Fluently.Configure()
                         .Database(MsSqlConfiguration.MsSql2008.ConnectionString(c => c
                            .Server("localhost")
                            .Database("bg")
                            .Username("")
                            .Password("")).IsolationLevel(IsolationLevel.ReadUncommitted));
                default: return Fluently.Configure()
                        .Database(SQLiteConfiguration.Standard
                            .UsingFile(DbFile));
            }
        }

        public void AddMappingInformation<T>()
        {
            config.Mappings(m => m.FluentMappings.AddFromAssemblyOf<T>());
        }

        public Persistenz Build()
        {
            config.BuildConfiguration().Properties.Add("current_session_context_class", "NHibernate.Context.CallSessionContext");
            if (mode == DBStartMode.DBÜberschreiben)
            {
                if (typ == DBTyp.SQLite)
                    config = config.ExposeConfiguration(BuildSchema1);
                else
                    config = config.ExposeConfiguration(BuildSchema);
            }
            sessionFactory = config.BuildSessionFactory();
            return this;
        }

        private void BuildSchema(Configuration config)
        {
            new SchemaExport(config)
                .Create(false, true);
        }

        private void BuildSchema1(Configuration config)
        {
            if (File.Exists(DbFile))
                File.Delete(DbFile);

            new SchemaExport(config)
                .Create(false, true);
        }

        public T Save<T>(T obj) where T : class
        {
            sessionFactory.GetCurrentSession().SaveOrUpdate(obj);
            return obj;
        }

        public IList<T> GetAll<T>() where T : class
        {
            return sessionFactory.GetCurrentSession().CreateCriteria<T>().List<T>();
        }

        public T GetById<T, I>(I id) where T : class
        {
            return sessionFactory.GetCurrentSession().Get<T>(id);
        }

        public void Delete<T>(T obj) where T : class
        {
            sessionFactory.GetCurrentSession().Delete(obj);
        }

        public IQueryable<T> LinqQuery<T>() where T : class
        {
            return sessionFactory.GetCurrentSession().Linq<T>();
        }

        public IConversation NewConversation()
        {
            return new MyConversation(sessionFactory);
        }
    }

    public enum DBTyp
    {
        MSSQL2005,
        MSSQL2008,
        SQLite
    }

    public enum DBStartMode
    {
        DBÜberschreiben,
        VorhandeneDBVerwenden
    }

    public sealed class MyConversation : IConversation
    {
        private readonly ISessionFactory sessionFactory;
        private ISession session;

        internal MyConversation(ISessionFactory sessionFactory)
        {
            this.sessionFactory = sessionFactory;
            this.session = null;
            Reset();
        }

        public void ExecuteTransactional(TransactionalCode body)
        {
            ITransaction transaction = null;
            try
            {
                lock (Persistenz.Transaction)
                {
                    transaction = session.BeginTransaction();
                    NHibernate.Context.CallSessionContext.Bind(session);
                    body();
                    transaction.Commit();
                }
            }
            catch (Exception ex)
            {
                try
                {
                    transaction.Rollback();
                }
                catch (Exception)
                {
                }
                throw ex;
            }
            finally
            {
                NHibernate.Context.CallSessionContext.Unbind(this.session.SessionFactory);
            }
        }


        public void Reset()
        {
            if (this.session != null)
            {
                try
                {
                    End();
                }
                catch (Exception)
                {
                }                
            }
            this.session = sessionFactory.OpenSession();
        }


        public void End()
        {
            this.session.Close();
        }

        public void Dispose()
        {
            End();
        }
    }
}
