﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using AnzeigenApp;
using BasisKlassen;
using Common.Logger;
using Service;
using Service.BoardService;

namespace ProfilApp
{
    /// <summary>
    /// Interaktionslogik für MeinProfilView.xaml
    /// </summary>
    public partial class MeinProfilView : MyUserControl
    {
        private Model model;
        private Profil profil;

        public MeinProfilView(Model model)
        {
            InitializeComponent();
            DataContext = this;
            this.model = model;
            Title = "MEIN PROFIL";
        }

        private void UserControl_Loaded_1(object sender, RoutedEventArgs e)
        {
            Logger.LogEvent(LogEvent.EigenesProfilAufgerufen, model.Guid);
            firstView.Visibility = Visibility.Visible;
            secondView.Visibility = Visibility.Hidden;
            try
            {
                var proxy = new Proxy();
                updateData(proxy.GetProfil(model.Guid));
            }
            catch (Exception ex) { }
        }

        private void updateData(Profil profil)
        {
            this.profil = profil;
            txtName.Text = (String.IsNullOrEmpty(profil.Nickname) ? profil.RegisteredEmail : profil.Nickname).ToUpper();
            txtNickname.Text = profil.Nickname;
            imgAvatar.Source = BitmapLoader.GetImageSource(profil.Profilbild);
            txtVorname.Text = profil.KontaktDetails.Vorname;
            txtNachname.Text = profil.KontaktDetails.Nachname;
            txtStrasse.Text = profil.KontaktDetails.Straße;
            txtPlz.Text = profil.KontaktDetails.Plz;
            txtStadt.Text = profil.KontaktDetails.Stadt;
            txtEmail.Text = profil.KontaktDetails.Email;
            txtTelefon.Text = profil.KontaktDetails.Telefon;
            chkName.IsChecked = profil.KontaktDetails.NameVisible;
            chkAdresse.IsChecked = profil.KontaktDetails.AdresseVisible;
            chkMail.IsChecked = profil.KontaktDetails.EmailVisible;
            chkTelefon.IsChecked = profil.KontaktDetails.TelefonVisible;
            chkSichtbar.IsChecked = profil.Sichtbar;
        }

        private void btnSave_Click_1(object sender, RoutedEventArgs e)
        {
            bool result = false;
            try
            {
                Logger.LogEvent(LogEvent.ProfilGespeichert, model.Guid);
                var proxy = new Proxy();
                var profil = proxy.ProfilAktualisieren(model.Guid, txtNickname.Text, txtVorname.Text, txtNachname.Text, chkName.IsChecked.Value, txtStrasse.Text, txtPlz.Text, txtStadt.Text,
                    chkAdresse.IsChecked.Value, txtTelefon.Text, chkTelefon.IsChecked.Value, txtEmail.Text, chkMail.IsChecked.Value, -1, chkSichtbar.IsChecked.Value);
                if (profil != null)
                    result = true;
                updateData(proxy.GetProfil(model.Guid));
            }
            catch (Exception ex) { }
            if (result)
                resultText.Text = "Änderungen erfolgreich übernommen";
            else
                resultText.Text = "Irgendetwas ist schiefgegangen\r\nDeine Änderungen wurden leider nicht übernommen";
            secondView.Visibility = Visibility.Visible;
            firstView.Visibility = Visibility.Hidden;
        }

        private void btnDelete_Click_1(object sender, RoutedEventArgs e)
        {
            try
            {
                Logger.LogEvent(LogEvent.ProfilbildGelöscht, model.Guid);
                var proxy = new Proxy();
                proxy.ProfilAktualisieren(model.Guid, profil.Nickname, profil.KontaktDetails.Vorname, profil.KontaktDetails.Nachname, profil.KontaktDetails.NameVisible,
                    profil.KontaktDetails.Straße, profil.KontaktDetails.Plz, profil.KontaktDetails.Stadt, profil.KontaktDetails.AdresseVisible, profil.KontaktDetails.Telefon,
                    profil.KontaktDetails.TelefonVisible, profil.KontaktDetails.Email, profil.KontaktDetails.EmailVisible, 0, profil.Sichtbar);
                updateData(proxy.GetProfil(model.Guid));
            }
            catch (Exception ex) { }
        }

        private void SurfaceButton_Click_1(object sender, RoutedEventArgs e)
        {
            firstView.Visibility = Visibility.Visible;
            secondView.Visibility = Visibility.Hidden;
        }
    }
}
