﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using FluentNHibernate.Mapping;

namespace ProfilKomponente
{
    [DataContract(Namespace = "http://marius-klausa.eu/board")]
    public class KontaktDetails
    {
        public virtual int ID { get; private set; }
        [DataMember(IsRequired = false)]
        public virtual String Vorname { get; set; }
        [DataMember(IsRequired = false)]
        public virtual String Nachname { get; set; }
        [DataMember(IsRequired = false)]
        public virtual bool NameVisible { get; set; }
        [DataMember(IsRequired = false)]
        public virtual String Straße { get; set; }
        [DataMember(IsRequired = false)]
        public virtual String Plz { get; set; }
        [DataMember(IsRequired = false)]
        public virtual String Stadt { get; set; }
        [DataMember]
        public virtual bool AdresseVisible { get; set; }
        [DataMember(IsRequired = false)]
        public virtual String Telefon { get; set; }
        [DataMember]
        public virtual bool TelefonVisible { get; set; }
        [DataMember(IsRequired = false)]
        public virtual String Email { get; set; }
        [DataMember]
        public virtual bool EmailVisible { get; set; }

        public virtual KontaktDetails GetVisibleKontaktDetails()
        {
            KontaktDetails clone = this.MemberwiseClone() as KontaktDetails;
            if (!NameVisible && !AdresseVisible)
            {
                clone.Vorname = null;
                clone.Nachname = null;
            }
            if (!TelefonVisible)
                clone.Telefon = null;
            if (!EmailVisible)
                clone.Email = null;
            if (!AdresseVisible)
            {
                clone.Straße = null;
                clone.Plz = null;
                clone.Stadt = null;
            }
            return clone;
        }

        public virtual KontaktDetails GetVisibleKontaktDetails(KontaktOptionen kontaktOptionen)
        {
            KontaktDetails clone = this.MemberwiseClone() as KontaktDetails;
            if (!kontaktOptionen.HasFlag(KontaktOptionen.Adresse) && !AdresseVisible)
            {
                if (!NameVisible)
                {
                    clone.Vorname = null;
                    clone.Nachname = null;
                }
                clone.Straße = null;
                clone.Plz = null;
                clone.Stadt = null;
            }
            if (!kontaktOptionen.HasFlag(KontaktOptionen.Telefon) && !TelefonVisible)
                clone.Telefon = null;
            if (!kontaktOptionen.HasFlag(KontaktOptionen.Email) && !EmailVisible)
                clone.Email = null;
            return clone;
        }

        public virtual string toHTMLString()
        {
            return "Kontakt:";
        }
    }

    public class KontaktDetailsMap : ClassMap<KontaktDetails>
    {
        public KontaktDetailsMap()
        {
            Id(x => x.ID);
            Map(x => x.Vorname);
            Map(x => x.Nachname);
            Map(x => x.NameVisible);
            Map(x => x.Straße);
            Map(x => x.Plz);
            Map(x => x.Stadt);
            Map(x => x.AdresseVisible);
            Map(x => x.Telefon);
            Map(x => x.TelefonVisible);
            Map(x => x.Email);
            Map(x => x.EmailVisible);
        }
    }
}
