﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Service.BoardService;

namespace Service
{
    public class Proxy
    {
        private IFassadeSOAPInterface fassade;
        private static String BOARD_GUID = "e89473ac-7dab-4ce5-b740-23f7a810cde4";
        public Proxy()
        {
            fassade = new FassadeSOAPInterfaceClient();
        }
        public string Registrieren(string email, string passwort)
        {
            try
            {
                return fassade.Registrieren(email, passwort);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public string Login(string email, string passwort)
        {
            try
            {
                return fassade.Login(email, passwort);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public bool AccountAktivieren(string guid, string code)
        {
            try
            {
                return fassade.AccountAktivieren(guid, code);
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public string GetAccountMail(string guid)
        {
            try
            {
                return fassade.GetAccountMail(guid);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public bool SendeAktivierungsmail(string guid)
        {
            try
            {
                return fassade.SendeAktivierungsmail(guid);
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public bool IstAccountAktiviert(string guid)
        {
            try
            {
                return fassade.IstAccountAktiviert(guid);
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public Profil GetProfil(string guid)
        {
            try
            {
                return fassade.GetProfil(guid);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public Profil GetProfilByRegisteredMail(string guid, string registeredEmail)
        {
            try
            {
                if (String.IsNullOrEmpty(guid))
                    guid = BOARD_GUID;
                return fassade.GetProfilByRegisteredMail(guid, registeredEmail);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public ProfilVorschau[] GetAlleProfile(string guid)
        {
            try
            {
                if (String.IsNullOrEmpty(guid))
                    guid = BOARD_GUID;
                return fassade.GetAlleProfile(guid);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public Profil ProfilAktualisieren(string guid, string nickname, string vorname, string nachname, bool nameVisible, string strasse, string plz, string stadt, bool adresseVisible, string telefon, bool telefonVisible, string email, bool emailVisible, int profilbildImageID, bool sichtbar)
        {
            try
            {
                return fassade.ProfilAktualisieren(guid, nickname, vorname, nachname, nameVisible, strasse, plz, stadt, adresseVisible, telefon, telefonVisible, email, emailVisible, profilbildImageID, sichtbar);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public Anzeige AnzeigeErstellen(string guid, string titel, string beschreibung, string text, Tag[] tags, Dictionary<int, int> kategorie, string appearance, int[] images, KontaktOptionen kontaktOptionen)
        {
            try
            {
                TransportAnzeige tmp = fassade.AnzeigeErstellen(guid, titel, beschreibung, text, tags, kategorie, appearance, images, kontaktOptionen);
                return tmp != null ? new Anzeige(tmp) : null;
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public Anzeige AnzeigeAktualisieren(string guid, int anzeigeID, string titel, string beschreibung, string text, Tag[] tags, Dictionary<int, int> kategorie, string appearance, int[] images)
        {
            try
            {
                TransportAnzeige tmp = fassade.AnzeigeAktualisieren(guid, anzeigeID, titel, beschreibung, text, tags, kategorie, appearance, images);
                return new Anzeige(tmp);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public bool AnzeigeSetÖffentlichStatus(string guid, int anzeigeID, bool status)
        {
            try
            {
                return fassade.AnzeigeSetÖffentlichStatus(guid, anzeigeID, status);
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public bool AnzeigeLöschen(string guid, int anzeigeID)
        {
            try
            {
                return fassade.AnzeigeLöschen(guid, anzeigeID);
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public Tag[] GetAllTags()
        {
            try
            {
                return fassade.GetAllTags();
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public Kategorie[] GetAllKategorien()
        {
            try
            {
                return fassade.GetAllKategorien();
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public Anzeige[] GetAllAnzeigen(string guid, int chunk)
        {
            try
            {
                if (String.IsNullOrEmpty(guid))
                    guid = BOARD_GUID;
                List<Anzeige> tmp = new List<Anzeige>();
                foreach (TransportAnzeige anz in fassade.GetAllAnzeigen(guid, chunk))
                    tmp.Add(new Anzeige(anz));
                return tmp.ToArray();
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public Anzeige[] GetOwnAnzeigen(string guid, int chunk)
        {
            try
            {
                List<Anzeige> tmp = new List<Anzeige>();
                foreach (TransportAnzeige anz in fassade.GetOwnAnzeigen(guid, chunk))
                    tmp.Add(new Anzeige(anz));
                return tmp.ToArray();
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public Anzeige GetAnzeige(string guid, int anzeigeID)
        {
            try
            {
                if (String.IsNullOrEmpty(guid))
                    guid = BOARD_GUID;
                TransportAnzeige tmp = fassade.GetAnzeige(guid, anzeigeID);
                return new Anzeige(tmp);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public bool SendeAnzeigePerMail(string guid, int anzeigeID)
        {
            try
            {
                return fassade.SendeAnzeigePerMail(guid, anzeigeID);
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public Anzeige[] GetAllAnzeigenFromUser(string guid, string nickname, int chunk)
        {
            try
            {
                if (String.IsNullOrEmpty(guid))
                    guid = BOARD_GUID;
                List<Anzeige> tmp = new List<Anzeige>();
                foreach (TransportAnzeige anz in fassade.GetAllAnzeigenFromUser(guid, nickname, chunk))
                    tmp.Add(new Anzeige(anz));
                return tmp.ToArray();
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public string GetImageHash(string guid, int imageID)
        {
            try
            {
                return fassade.GetImageHash(guid, imageID);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public string GetImagePath(string guid, int imageID)
        {
            try
            {
                return fassade.GetImagePath(guid, imageID);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public string[] GetImagePaths(string guid, int[] imageids)
        {
            try
            {
                return fassade.GetImagePaths(guid, imageids);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public bool IstOwnAnzeige(string guid, int anzeigeID)
        {
            try
            {
                return fassade.IstOwnAnzeige(guid, anzeigeID);
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public FileTransferResult ImageErstellen(FileTransferRequest request)
        {
            try
            {
                return fassade.ImageErstellen(request);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public bool PostFeedback(string guid, string text)
        {
            try
            {
                return fassade.PostFeedback(guid, text);
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public Feedback[] GetFeedbackList(string guid, int chunk)
        {
            try
            {
                return fassade.GetFeedbackList(guid, chunk);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public Profil GetProfilByID(string guid, int profileAccountID)
        {
            try
            {
                if (String.IsNullOrEmpty(guid))
                    guid = BOARD_GUID;
                return fassade.GetProfilByID(guid, profileAccountID);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public void IsAlive(string guid)
        {
            try
            {
                guid = BOARD_GUID;
                fassade.IsAlive(guid);
            }
            catch (Exception e)
            {
            }
        }

        public bool Status()
        {
            try
            {
                return fassade.Status();
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public void UploadLogfile(FileTransferRequest request)
        {
            try
            {
                request.guid = BOARD_GUID;
                fassade.UploadLogfile(request);
            }
            catch (Exception e)
            {
            }
        }

        public bool ChangePassword(string guid, string guidToChange, string oldPW, string newPW)
        {
            try
            {
                return fassade.ChangePassword(guid, guidToChange, oldPW, newPW);
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public bool DeleteKategorie(string guid, int kategorieID)
        {
            try
            {
                return fassade.DeleteKategorie(guid, kategorieID);
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public Kategorie CreateKategorie(string guid, string name, string[] unterkategorien)
        {
            try
            {
                return fassade.CreateKategorie(guid, name, unterkategorien);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public string GetGuid(string guid, string email)
        {
            try
            {
                return fassade.GetGuid(guid, email);
            }
            catch (Exception e)
            {
                return null;
            }
        }

        public string CreateAccount(string guid, string email, string passwort, bool activated)
        {
            try
            {
                return fassade.CreateAccount(guid, email, passwort, activated);
            }
            catch (Exception e)
            {
                return null;
            }
        }
    }
}
