﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;
using PersistenzManager;
using Fassade;
using SitzungsVerwaltungsKomponente;
using RechteVerwaltungsKomponente;
using AccountVerwaltungsKomponente;
using AnzeigenKomponente;
using BilderKomponente;
using System.ServiceModel.Channels;
using ProfilKomponente;
using FeedbackKomponente;
using Common;

namespace WCF_Service
{
    class Program
    {
        public static IÖffentlicheFassade fassade;
        static void Main(string[] args)
        {
            Persistenz tmpPersistenz = new Persistenz(DBTyp.SQLite, DBStartMode.VorhandeneDBVerwenden);
            tmpPersistenz.AddMappingInformation<Berechtigung>();
            tmpPersistenz.AddMappingInformation<Aktivierung>();
            tmpPersistenz.AddMappingInformation<Anzeige>();
            tmpPersistenz.AddMappingInformation<Image>();
            tmpPersistenz.AddMappingInformation<Profil>();
            tmpPersistenz.AddMappingInformation<Feedback>();
            tmpPersistenz = tmpPersistenz.Build();

            IPersistenz persistenz = tmpPersistenz as IPersistenz;
            IConversationFactory factory = tmpPersistenz as IConversationFactory;

            Program.fassade = new ÖffentlicheFassade(persistenz, factory);
            //ErzeugeTestdaten(persistenz, factory, fassade);

            //SitzungsVerwaltung.Start();



            Uri baseAddress = new Uri(AppConfHelper.GetValueAsString("hosting_url"));

            ServiceHost host = new ServiceHost(typeof(ServiceFassade), baseAddress);

            BasicHttpBinding binding = new BasicHttpBinding();
            binding.MaxReceivedMessageSize = 1000000000;
            //binding.Namespace = "http://marius-klausa.eu/board";


            WebHttpBinding webBinding = new WebHttpBinding();
            webBinding.MaxReceivedMessageSize = 1000000000;

            ServiceEndpoint sep = host.AddServiceEndpoint(
                typeof(IFassadeSOAPInterface),
                binding,
                "boardService");

            ServiceEndpoint sep2 = host.AddServiceEndpoint(
               typeof(IFassadeWebInterface),
               webBinding,
               "board");
            WebHttpBehavior whb = sep2.Behaviors.Find<WebHttpBehavior>();


            WebHttpBehavior webBehavior = new WebHttpBehavior();
            webBehavior.AutomaticFormatSelectionEnabled = true;
            sep2.Behaviors.Add(webBehavior);

            ServiceMetadataBehavior smb = new ServiceMetadataBehavior();
            smb.HttpGetEnabled = true;
            host.Description.Behaviors.Add(smb);


            host.Open();

            Console.WriteLine("Service is running");
            Console.WriteLine("--- Press any Key to exit ---");
            Console.ReadKey();
            host.Close();
        }

        private static void ErzeugeTestdaten(IPersistenz persistenz, IConversationFactory conversationFactory, IÖffentlicheFassade fassade)
        {
            using (var conversation = conversationFactory.NewConversation())
            {
                conversation.ExecuteTransactional(new TransactionalCode(() =>
                {
                    Berechtigung b1 = new Berechtigung(AccountType.Inaktiv, Anwendungsfall.InaktivUserFall);
                    persistenz.Save(b1);
                    b1 = new Berechtigung(AccountType.Aktiv, Anwendungsfall.AktivUserFall);
                    persistenz.Save(b1);
                    b1 = new Berechtigung(AccountType.Board, Anwendungsfall.BoardFall);
                    persistenz.Save(b1);
                    b1 = new Berechtigung(AccountType.Admin, Anwendungsfall.AdminFall);
                    persistenz.Save(b1);
                }));
            }
            Kategorie Design = null;
            Kategorie Medien = null;
            Kategorie Gesundheit = null;
            Kategorie Umwelt = null;
            Kategorie Fahrzeugbau = null;
            Kategorie Flugzeugbau = null;
            Kategorie Informatik = null;
            Kategorie Elektrotechnik = null;
            Kategorie Maschinenbau = null;
            Kategorie Produktion = null;

            Kategorie Englisch = null;
            Kategorie Spanisch = null;
            Kategorie Chinesisch = null;
            Kategorie Deutsch = null;
            Kategorie Hindi = null;
            Kategorie Arabisch = null;
            Kategorie Russisch = null;
            Kategorie Portugiesisch = null;
            Kategorie Japanisch = null;
            Kategorie Französisch = null;

            Kategorie Andere = null;
            Kategorie Jobs = null;

            Kategorie Sonstiges = null;
            Kategorie Mitfahrgelegenheit = null;
            Kategorie Sprachpartner = null;
            Kategorie Nachhilfe = null;
            Kategorie Buch = null;
            Kategorie Fachbuch = null;
            Kategorie Roman = null;
            Kategorie Wohnen = null;
            Kategorie AutoRad = null;
            Kategorie Multimedia = null;

            Kategorie Angebot = null;
            Kategorie Gesuch = null;
            using (var conversation = conversationFactory.NewConversation())
            {
                conversation.ExecuteTransactional(new TransactionalCode(() =>
                {
                    Design = new Kategorie("Design");
                    Medien = new Kategorie("Medien");
                    Gesundheit = new Kategorie("Gesundheit");
                    Umwelt = new Kategorie("Umwelt");
                    Fahrzeugbau = new Kategorie("Fahrzeugbau");
                    Flugzeugbau = new Kategorie("Flugzeugbau");
                    Informatik = new Kategorie("Informatik");
                    Elektrotechnik = new Kategorie("Elektrotechnik");
                    Maschinenbau = new Kategorie("Maschinenbau");
                    Produktion = new Kategorie("Produktion");
                    Andere = new Kategorie("Andere");
                    IList<Kategorie> subNachhilfe = new List<Kategorie>() { Design, Medien, Gesundheit, Umwelt, Fahrzeugbau, Flugzeugbau, Informatik, Elektrotechnik, Maschinenbau, Produktion, Andere };
                    Englisch = new Kategorie("Englisch");
                    Spanisch = new Kategorie("Spanisch");
                    Chinesisch = new Kategorie("Chinesisch");
                    Deutsch = new Kategorie("Deutsch");
                    Hindi = new Kategorie("Hindi");
                    Arabisch = new Kategorie("Arabisch");
                    Russisch = new Kategorie("Russisch");
                    Portugiesisch = new Kategorie("Portugiesisch");
                    Japanisch = new Kategorie("Japanisch");
                    Französisch = new Kategorie("Französisch");

                    IList<Kategorie> subSprachen = new List<Kategorie>() { Englisch, Spanisch, Chinesisch, Deutsch, Hindi, Arabisch, Russisch, Portugiesisch, Japanisch, Französisch, Andere };

                    Nachhilfe = new Kategorie("Nachhilfe", subNachhilfe);
                    Sprachpartner = new Kategorie("Sprachpartner", subSprachen);

                    Sonstiges = new Kategorie("Sonstiges");
                    AutoRad = new Kategorie("Auto & Fahrrad");
                    Mitfahrgelegenheit = new Kategorie("Mitfahrgelegenheit");
                    Fachbuch = new Kategorie("Fachbuch");
                    Roman = new Kategorie("Roman");
                    IList<Kategorie> subBuch = new List<Kategorie>() { Fachbuch, Roman, Andere };
                    Buch = new Kategorie("Buch", subBuch);
                    Wohnen = new Kategorie("Wohnen");
                    Jobs = new Kategorie("Jobs");
                    Multimedia = new Kategorie("Multimedia & Elektronik");

                    IList<Kategorie> subAngebot = new List<Kategorie>() { Wohnen, Buch, Nachhilfe, Sprachpartner, Mitfahrgelegenheit, AutoRad, Jobs, Multimedia, Sonstiges };
                    IList<Kategorie> subGesuch = new List<Kategorie>() { Wohnen, Buch, Nachhilfe, Sprachpartner, Mitfahrgelegenheit, AutoRad, Jobs, Multimedia, Sonstiges };

                    Angebot = new Kategorie("Angebot", subAngebot);
                    Gesuch = new Kategorie("Gesuch", subGesuch);
                    persistenz.Save<Kategorie>(Design);//1
                    persistenz.Save<Kategorie>(Medien);//2
                    persistenz.Save<Kategorie>(Gesundheit);//3
                    persistenz.Save<Kategorie>(Umwelt);//4
                    persistenz.Save<Kategorie>(Fahrzeugbau);//5
                    persistenz.Save<Kategorie>(Flugzeugbau);//6
                    persistenz.Save<Kategorie>(Informatik);//7
                    persistenz.Save<Kategorie>(Elektrotechnik);//8
                    persistenz.Save<Kategorie>(Maschinenbau);//9
                    persistenz.Save<Kategorie>(Produktion);//10
                    persistenz.Save<Kategorie>(Englisch);//11
                    persistenz.Save<Kategorie>(Spanisch);//12
                    persistenz.Save<Kategorie>(Nachhilfe);//13
                    persistenz.Save<Kategorie>(Sprachpartner);//14
                    persistenz.Save<Kategorie>(Sonstiges);//15
                    persistenz.Save<Kategorie>(Mitfahrgelegenheit);//16
                    persistenz.Save<Kategorie>(Buch);//17
                    persistenz.Save<Kategorie>(Wohnen);//18
                    persistenz.Save<Kategorie>(Angebot);//19
                    persistenz.Save<Kategorie>(Gesuch);//20

                    persistenz.Save<Kategorie>(Chinesisch);//21
                    persistenz.Save<Kategorie>(Hindi);//22
                    persistenz.Save<Kategorie>(Arabisch);//23
                    persistenz.Save<Kategorie>(Russisch);//24
                    persistenz.Save<Kategorie>(Portugiesisch);//25
                    persistenz.Save<Kategorie>(Japanisch);//26
                    persistenz.Save<Kategorie>(Deutsch);//27
                    persistenz.Save<Kategorie>(Französisch);//28
                    persistenz.Save<Kategorie>(Andere);//29
                    persistenz.Save<Kategorie>(Jobs);//30
                    persistenz.Save<Kategorie>(Fachbuch);//31
                    persistenz.Save<Kategorie>(Roman);//32
                    persistenz.Save<Kategorie>(AutoRad);//33
                    persistenz.Save<Kategorie>(Multimedia);//34
                }));
            }

            string adminGuid = fassade.Registrieren("admin", "adminPW");
            fassade.Registrieren("board", "boardPW");
            using (var conversation = conversationFactory.NewConversation())
            {
                conversation.ExecuteTransactional(new TransactionalCode(() =>
                {
                    Account acc = persistenz.LinqQuery<Account>().Where(x => x.Email.Equals("admin")).SingleOrDefault();
                    acc.Status = AccountType.Admin;
                    persistenz.Save<Account>(acc);
                }));
            }
            using (var conversation = conversationFactory.NewConversation())
            {
                conversation.ExecuteTransactional(new TransactionalCode(() =>
                {
                    Account acc = persistenz.LinqQuery<Account>().Where(x => x.Email.Equals("board")).SingleOrDefault();
                    acc.Status = AccountType.Board;
                    persistenz.Save<Account>(acc);
                }));
            }
        }
    }
}
